@@ -1,18 +1,18 @@
-"""
-A reader that drain a stream via its file no on a background thread.
-"""
+"""A reader that drain a stream via its file no on a background thread."""
 from __future__ import annotations
 
 from abc import ABC, abstractmethod
 from threading import Event, Thread
-from types import TracebackType
-from typing import Callable
+from typing import TYPE_CHECKING, Callable
+
+if TYPE_CHECKING:
+    from types import TracebackType
 
 WAIT_GENERAL = 0.05  # stop thread join every so often (give chance to a signal interrupt)
 
 
 class ReadViaThread(ABC):
-    def __init__(self, file_no: int, handler: Callable[[bytes], None], name: str, drain: bool) -> None:
+    def __init__(self, file_no: int, handler: Callable[[bytes], None], name: str, drain: bool) -> None:  # noqa: FBT001
         self.file_no = file_no
         self.stop = Event()
         self.thread = Thread(target=self._read_stream, name=f"tox-r-{name}-{file_no}")
@@ -25,9 +25,9 @@ class ReadViaThread(ABC):
 
     def __exit__(
         self,
-        exc_type: type[BaseException] | None,  # noqa: U100
-        exc_val: BaseException | None,  # noqa: U100
-        exc_tb: TracebackType | None,  # noqa: U100
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
     ) -> None:
         self.stop.set()  # signal thread to stop
         while self.thread.is_alive():  # wait until it stops
