@@ -5,22 +5,19 @@ import os
 import sys
 from collections import defaultdict
 from contextlib import contextmanager
+from itertools import chain
 from pathlib import Path
 from threading import RLock
-from typing import Any, Dict, Generator, Iterator, NoReturn, Optional, Sequence, cast
+from typing import TYPE_CHECKING, Any, Dict, Generator, Iterator, NoReturn, Optional, Sequence, cast
 
 from cachetools import cached
 from packaging.requirements import Requirement
 from pyproject_api import BackendFailed, CmdStatus, Frontend
 
-from tox.config.sets import EnvConfigSet
-from tox.execute.api import ExecuteStatus
 from tox.execute.pep517_backend import LocalSubProcessPep517Executor
 from tox.execute.request import StdinSource
 from tox.plugin import impl
-from tox.tox_env.api import ToxEnvCreateArgs
 from tox.tox_env.errors import Fail
-from tox.tox_env.package import Package, PackageToxEnv
 from tox.tox_env.python.package import (
     EditableLegacyPackage,
     EditablePackage,
@@ -28,12 +25,18 @@ from tox.tox_env.python.package import (
     SdistPackage,
     WheelPackage,
 )
-from tox.tox_env.register import ToxEnvRegister
-from tox.tox_env.runner import RunToxEnv
+from tox.tox_env.python.virtual_env.api import VirtualEnv
 from tox.util.file_view import create_session_view
 
-from ..api import VirtualEnv
-from .util import dependencies_with_extras
+from .util import dependencies_with_extras, dependencies_with_extras_from_markers
+
+if TYPE_CHECKING:
+    from tox.config.sets import EnvConfigSet
+    from tox.execute.api import ExecuteStatus
+    from tox.tox_env.api import ToxEnvCreateArgs
+    from tox.tox_env.package import Package, PackageToxEnv
+    from tox.tox_env.register import ToxEnvRegister
+    from tox.tox_env.runner import RunToxEnv
 
 if sys.version_info >= (3, 8):  # pragma: no cover (py38+)
     from importlib.metadata import Distribution, PathDistribution
@@ -64,8 +67,8 @@ class ToxBackendFailed(Fail, BackendFailed):
         )
 
 
-class BuildEditableNotSupported(RuntimeError):
-    """raised when build editable is not supported"""
+class BuildEditableNotSupportedError(RuntimeError):
+    """raised when build editable is not supported."""
 
 
 class ToxCmdStatus(CmdStatus):
@@ -89,12 +92,13 @@ class ToxCmdStatus(CmdStatus):
 
 
 class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
-    """local file system python virtual environment via the virtualenv package"""
+    """local file system python virtual environment via the virtualenv package."""
 
     def __init__(self, create_args: ToxEnvCreateArgs) -> None:
         super().__init__(create_args)
         self._frontend_: Pep517VirtualEnvFrontend | None = None
         self.builds: defaultdict[str, list[EnvConfigSet]] = defaultdict(list)
+        self.call_require_hooks: set[str] = set()
         self._distribution_meta: PathDistribution | None = None
         self._package_dependencies: list[Requirement] | None = None
         self._package_name: str | None = None
@@ -103,7 +107,7 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
         self._package_paths: set[Path] = set()
 
     @staticmethod
-    def id() -> str:
+    def id() -> str:  # noqa: A003
         return "virtualenv-pep-517"
 
     @property
@@ -117,13 +121,13 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
         self.conf.add_config(
             keys=["meta_dir"],
             of_type=Path,
-            default=lambda conf, name: self.env_dir / ".meta",  # noqa: U100
+            default=lambda conf, name: self.env_dir / ".meta",  # noqa: ARG005
             desc="directory where to put the project metadata files",
         )
         self.conf.add_config(
             keys=["pkg_dir"],
             of_type=Path,
-            default=lambda conf, name: self.env_dir / "dist",  # noqa: U100
+            default=lambda conf, name: self.env_dir / "dist",  # noqa: ARG005
             desc="directory where to put project packages",
         )
 
@@ -137,31 +141,41 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
         meta_folder.mkdir(exist_ok=True)
         return meta_folder
 
+    @property
+    def meta_folder_if_populated(self) -> Path | None:
+        """Return the metadata directory if it contains any files, otherwise None."""
+        meta_folder = self.meta_folder
+        if meta_folder.exists() and tuple(meta_folder.iterdir()):
+            return meta_folder
+        return None
+
     def register_run_env(self, run_env: RunToxEnv) -> Generator[tuple[str, str], PackageToxEnv, None]:
         yield from super().register_run_env(run_env)
         build_type = run_env.conf["package"]
+        self.call_require_hooks.add(build_type)
         self.builds[build_type].append(run_env.conf)
 
     def _setup_env(self) -> None:
         super()._setup_env()
-        if "editable" in self.builds:
+        if "sdist" in self.call_require_hooks or "external" in self.call_require_hooks:
+            self._setup_build_requires("sdist")
+        if "wheel" in self.call_require_hooks:
+            self._setup_build_requires("wheel")
+        if "editable" in self.call_require_hooks:
             if not self._frontend.optional_hooks["build_editable"]:
-                raise BuildEditableNotSupported
-            build_requires = self._frontend.get_requires_for_build_editable().requires
-            self.installer.install(build_requires, PythonPackageToxEnv.__name__, "requires_for_build_editable")
-        if "wheel" in self.builds:
-            build_requires = self._frontend.get_requires_for_build_wheel().requires
-            self.installer.install(build_requires, PythonPackageToxEnv.__name__, "requires_for_build_wheel")
-        if "sdist" in self.builds or "external" in self.builds:
-            build_requires = self._frontend.get_requires_for_build_sdist().requires
-            self.installer.install(build_requires, PythonPackageToxEnv.__name__, "requires_for_build_sdist")
+                raise BuildEditableNotSupportedError
+            self._setup_build_requires("editable")
+
+    def _setup_build_requires(self, of_type: str) -> None:
+        requires = getattr(self._frontend, f"get_requires_for_build_{of_type}")().requires
+        self._install(requires, PythonPackageToxEnv.__name__, f"requires_for_build_{of_type}")
 
     def _teardown(self) -> None:
         executor = self._frontend.backend_executor
         if executor is not None:  # pragma: no branch
             try:
                 if executor.is_alive:
-                    self._frontend._send("_exit")  # try first on amicable shutdown
+                    self._frontend._send("_exit")  # try first on amicable shutdown  # noqa: SLF001
             except SystemExit:  # pragma: no cover  # if already has been interrupted ignore
                 pass
             finally:
@@ -173,24 +187,29 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
         super()._teardown()
 
     def perform_packaging(self, for_env: EnvConfigSet) -> list[Package]:
-        """build the package to install"""
+        """Build the package to install."""
         try:
             deps = self._load_deps(for_env)
-        except BuildEditableNotSupported:
+        except BuildEditableNotSupportedError:
+            self.call_require_hooks.remove("editable")
             targets = [e for e in self.builds.pop("editable") if e["package"] == "editable"]
             names = ", ".join(sorted({t.env_name for t in targets if t.env_name}))
-            logging.error(
-                f"package config for {names} is editable, however the build backend {self._frontend.backend}"
-                f" does not support PEP-660, falling back to editable-legacy - change your configuration to it",
+
+            logging.error(  # noqa: TRY400
+                "package config for %s is editable, however the build backend %s does not support PEP-660, falling "
+                "back to editable-legacy - change your configuration to it",
+                names,
+                cast(Pep517VirtualEnvFrontend, self._frontend_).backend,
             )
             for env in targets:
-                env._defined["package"].value = "editable-legacy"  # type: ignore
+                env._defined["package"].value = "editable-legacy"  # type: ignore[attr-defined]  # noqa: SLF001
                 self.builds["editable-legacy"].append(env)
+            self._run_state["setup"] = False  # force setup again as we need to provision wheel to get dependencies
             deps = self._load_deps(for_env)
         of_type: str = for_env["package"]
         if of_type == "editable-legacy":
             self.setup()
-            deps = [*self.requires(), *self._frontend.get_requires_for_build_sdist().requires] + deps
+            deps = [*self.requires(), *self._frontend.get_requires_for_build_sdist().requires, *deps]
             package: Package = EditableLegacyPackage(self.core["tox_root"], deps)  # the folder itself is the package
         elif of_type == "sdist":
             self.setup()
@@ -210,14 +229,15 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
                 with self._pkg_lock:
                     wheel = getattr(self._frontend, method)(
                         wheel_directory=self.pkg_dir,
-                        metadata_directory=self.meta_folder,
+                        metadata_directory=self.meta_folder_if_populated,
                         config_settings=self._wheel_config_settings,
                     ).wheel
                     wheel = create_session_view(wheel, self._package_temp_path)
                     self._package_paths.add(wheel)
                 package = (EditablePackage if of_type == "editable" else WheelPackage)(wheel, deps)
         else:  # pragma: no cover # for when we introduce new packaging types and don't implement
-            raise TypeError(f"cannot handle package type {of_type}")  # pragma: no cover
+            msg = f"cannot handle package type {of_type}"
+            raise TypeError(msg)  # pragma: no cover
         return [package]
 
     @property
@@ -245,11 +265,17 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
             if dynamic == "dependencies" or (extras and dynamic == "optional-dependencies"):
                 return None  # if any dependencies are dynamic we can just calculate all dynamically
 
-        deps: list[Requirement] = [Requirement(i) for i in project.get("dependencies", [])]
+        deps_with_markers: list[tuple[Requirement, set[str | None]]] = [
+            (Requirement(i), {None}) for i in project.get("dependencies", [])
+        ]
         optional_deps = project.get("optional-dependencies", {})
-        for extra in extras:
-            deps.extend(Requirement(i) for i in optional_deps.get(extra, []))
-        return deps
+        for extra, reqs in optional_deps.items():
+            deps_with_markers.extend((Requirement(req), {extra}) for req in (reqs or []))
+        return dependencies_with_extras_from_markers(
+            deps_with_markers=deps_with_markers,
+            extras=extras,
+            package_name=project.get("name", "."),
+        )
 
     def _load_deps_from_built_metadata(self, for_env: EnvConfigSet) -> list[Requirement]:
         # dependencies might depend on the python environment we're running in => if we build a wheel use that env
@@ -269,8 +295,7 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
             reqs = self.get_package_dependencies(for_env)
             name = self.get_package_name(for_env)
         extras: set[str] = for_env["extras"]
-        deps = dependencies_with_extras(reqs, extras, name)
-        return deps
+        return dependencies_with_extras(reqs, extras, name)
 
     def get_package_dependencies(self, for_env: EnvConfigSet) -> list[Requirement]:
         with self._pkg_lock:
@@ -290,12 +315,13 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
     def _ensure_meta_present(self, for_env: EnvConfigSet) -> None:
         if self._distribution_meta is not None:  # pragma: no branch
             return  # pragma: no cover
+        # even if we don't build a wheel we need the requirements for it should we want to build its metadata
+        target = "editable" if for_env["package"] == "editable" else "wheel"
+        self.call_require_hooks.add(target)
+
         self.setup()
-        end = self._frontend
-        if for_env["package"] == "editable":
-            dist_info = end.prepare_metadata_for_build_editable(self.meta_folder, self._wheel_config_settings).metadata
-        else:
-            dist_info = end.prepare_metadata_for_build_wheel(self.meta_folder, self._wheel_config_settings).metadata
+        hook = getattr(self._frontend, f"prepare_metadata_for_build_{target}")
+        dist_info = hook(self.meta_folder, self._wheel_config_settings).metadata
         self._distribution_meta = Distribution.at(str(dist_info))
 
     @property
@@ -312,24 +338,30 @@ class Pep517VirtualEnvFrontend(Frontend):
         self._tox_env = env
         self._backend_executor_: LocalSubProcessPep517Executor | None = None
         into: dict[str, Any] = {}
-        pkg_cache = cached(
-            into,
-            key=lambda *args, **kwargs: "wheel" if "wheel_directory" in kwargs else "sdist",  # noqa: U100
-        )
-        self.build_wheel = pkg_cache(self.build_wheel)  # type: ignore
-        self.build_sdist = pkg_cache(self.build_sdist)  # type: ignore
-        self.build_editable = pkg_cache(self.build_editable)  # type: ignore
+
+        for hook in chain(
+            (f"get_requires_for_build_{build_type}" for build_type in ["editable", "wheel", "sdist"]),
+            (f"prepare_metadata_for_build_{build_type}" for build_type in ["editable", "wheel"]),
+            (f"build_{build_type}" for build_type in ["editable", "wheel", "sdist"]),
+        ):  # wrap build methods in a cache wrapper
+
+            def key(*args: Any, bound_return: str = hook, **kwargs: Any) -> str:  # noqa: ARG001
+                return bound_return
+
+            setattr(self, hook, cached(into, key=key)(getattr(self, hook)))
 
     @property
     def backend_cmd(self) -> Sequence[str]:
-        return ["python"] + self.backend_args
+        return ["python", *self.backend_args]
 
     def _send(self, cmd: str, **kwargs: Any) -> tuple[Any, str, str]:
         try:
-            if cmd in ("prepare_metadata_for_build_wheel", "prepare_metadata_for_build_editable"):
+            if (
+                cmd in ("prepare_metadata_for_build_wheel", "prepare_metadata_for_build_editable")
                 # given we'll build a wheel we might skip the prepare step
-                if "wheel" in self._tox_env.builds or "editable" in self._tox_env.builds:
-                    return None, "", ""  # will need to build wheel either way, avoid prepare
+                and ("wheel" in self._tox_env.builds or "editable" in self._tox_env.builds)
+            ):
+                return None, "", ""  # will need to build wheel either way, avoid prepare
             return super()._send(cmd, **kwargs)
         except BackendFailed as exception:
             raise exception if isinstance(exception, ToxBackendFailed) else ToxBackendFailed(exception) from exception
@@ -338,7 +370,7 @@ class Pep517VirtualEnvFrontend(Frontend):
     def _send_msg(
         self,
         cmd: str,
-        result_file: Path,  # noqa: U100
+        result_file: Path,  # noqa: ARG002
         msg: str,
     ) -> Iterator[ToxCmdStatus]:
         with self._tox_env.execute_async(
@@ -355,7 +387,14 @@ class Pep517VirtualEnvFrontend(Frontend):
         if outcome is not None:  # pragma: no branch
             outcome.assert_success()
 
-    def _unexpected_response(self, cmd: str, got: Any, expected_type: Any, out: str, err: str) -> NoReturn:
+    def _unexpected_response(  # noqa: PLR0913
+        self,
+        cmd: str,
+        got: Any,
+        expected_type: Any,
+        out: str,
+        err: str,
+    ) -> NoReturn:
         try:
             super()._unexpected_response(cmd, got, expected_type, out, err)
         except BackendFailed as exception:
