@@ -9,21 +9,25 @@ import subprocess
 import sys
 from io import TextIOWrapper
 from pathlib import Path
+from typing import TYPE_CHECKING
 from unittest.mock import MagicMock, create_autospec
 
 import psutil
 import pytest
 from colorama import Fore
 from psutil import AccessDenied
-from pytest_mock import MockerFixture
 
 from tox.execute.api import ExecuteOptions, Outcome
 from tox.execute.local_sub_process import SIG_INTERRUPT, LocalSubProcessExecuteInstance, LocalSubProcessExecutor
 from tox.execute.request import ExecuteRequest, StdinSource
 from tox.execute.stream import SyncWrite
-from tox.pytest import CaptureFixture, LogCaptureFixture, MonkeyPatch
 from tox.report import NamedBytesIO
 
+if TYPE_CHECKING:
+    from pytest_mock import MockerFixture
+
+    from tox.pytest import CaptureFixture, LogCaptureFixture, MonkeyPatch
+
 
 class FakeOutErr:
     def __init__(self) -> None:
@@ -38,7 +42,7 @@ class FakeOutErr:
 @pytest.mark.parametrize("color", [True, False], ids=["color", "no_color"])
 @pytest.mark.parametrize(("out", "err"), [("out", "err"), ("", "")], ids=["simple", "nothing"])
 @pytest.mark.parametrize("show", [True, False], ids=["show", "no_show"])
-def test_local_execute_basic_pass(
+def test_local_execute_basic_pass(  # noqa: PLR0913
     caplog: LogCaptureFixture,
     os_env: dict[str, str],
     out: str,
@@ -216,8 +220,8 @@ def test_command_does_not_exist(caplog: LogCaptureFixture, os_env: dict[str, str
 
     assert bool(outcome) is False, outcome
     assert outcome.exit_code != Outcome.OK
-    assert outcome.out == ""
-    assert outcome.err == ""
+    assert not outcome.out
+    assert not outcome.err
     assert not caplog.records
 
 
@@ -237,7 +241,7 @@ def test_command_keyboard_interrupt(tmp_path: Path, monkeypatch: MonkeyPatch, ca
         pytest.skip(str(exc))  # pragma: no cover
         raise  # pragma: no cover
 
-    print(f"test running in {os.getpid()} and sending CTRL+C to {process.pid}", file=sys.stderr)
+    print(f"test running in {os.getpid()} and sending CTRL+C to {process.pid}", file=sys.stderr)  # noqa: T201
     process.send_signal(SIG_INTERRUPT)
     try:
         process.communicate(timeout=3)
