@@ -1,8 +1,11 @@
 from __future__ import annotations
 
+from typing import TYPE_CHECKING
+
 import pytest
 
-from tox.pytest import ToxProjectCreator
+if TYPE_CHECKING:
+    from tox.pytest import ToxProjectCreator
 
 
 def test_devenv_fail_multiple_target(tox_project: ToxProjectCreator) -> None:
@@ -13,7 +16,7 @@ def test_devenv_fail_multiple_target(tox_project: ToxProjectCreator) -> None:
 
 
 @pytest.mark.integration()
-def test_devenv_ok(tox_project: ToxProjectCreator, enable_pip_pypi_access: str | None) -> None:  # noqa: U100
+def test_devenv_ok(tox_project: ToxProjectCreator, enable_pip_pypi_access: str | None) -> None:  # noqa: ARG001
     content = {
         "setup.py": "from setuptools import setup\nsetup(name='demo', version='1.0')",
         "tox.ini": "[tox]\nenv_list = py\n[testenv]\nusedevelop = True",
