@@ -5,17 +5,19 @@ import os
 import re
 import sys
 from pathlib import Path
-from typing import Any
+from typing import TYPE_CHECKING, Any
 
 import pytest
 from re_assert import Matches
 from virtualenv.discovery.py_info import PythonInfo
 
 from tox import __version__
-from tox.pytest import ToxProjectCreator
 from tox.tox_env.api import ToxEnv
 from tox.tox_env.info import Info
 
+if TYPE_CHECKING:
+    from tox.pytest import ToxProjectCreator
+
 
 @pytest.mark.parametrize("prefix", ["-", "- "])
 def test_run_ignore_cmd_exit_code(tox_project: ToxProjectCreator, prefix: str) -> None:
@@ -44,10 +46,20 @@ def test_run_sequential_fail(tox_project: ToxProjectCreator) -> None:
     assert Matches(r"  a: FAIL code 1 \(.*=setup\[.*\]\+cmd\[.*\] seconds\)") == reports[-3]
 
 
+def test_run_sequential_quiet(tox_project: ToxProjectCreator) -> None:
+    ini = "[tox]\nenv_list=a\nno_package=true\n[testenv]\ncommands=python -V"
+    project = tox_project({"tox.ini": ini})
+    outcome = project.run("r", "-q", "-e", "a")
+    outcome.assert_success()
+    reports = outcome.out.splitlines()[-3:]
+    assert Matches(r"  congratulations :\) \(.* seconds\)") == reports[-1]
+    assert Matches(r"  a: OK \([\d.]+ seconds\)") == reports[-2]
+
+
 @pytest.mark.integration()
 def test_result_json_sequential(
     tox_project: ToxProjectCreator,
-    enable_pip_pypi_access: str | None,  # noqa: U100
+    enable_pip_pypi_access: str | None,  # noqa: ARG001
 ) -> None:
     cmd = [
         "- python -c 'import sys; print(\"magic fail\", file=sys.stderr); sys.exit(1)'",
@@ -178,7 +190,7 @@ def test_package_deps_change(tox_project: ToxProjectCreator, demo_pkg_inline: Pa
     assert ".pkg: install" not in result_first.out  # no deps initially
 
     # new deps are picked up
-    (proj.path / "pyproject.toml").write_text(toml.replace("requires = []", 'requires = ["wheel"]'))
+    (proj.path / "pyproject.toml").write_text(toml.replace("requires = [\n]", 'requires = ["wheel"]'))
     (proj.path / "build.py").write_text(build.replace("return []", "return ['setuptools']"))
 
     result_rerun = proj.run("r")
@@ -280,7 +292,7 @@ def test_skip_pkg_install(tox_project: ToxProjectCreator, demo_pkg_inline: Path)
 def test_skip_develop_mode(tox_project: ToxProjectCreator, demo_pkg_setuptools: Path) -> None:
     proj = tox_project({"tox.ini": "[testenv]\npackage=wheel\n"})
     execute_calls = proj.patch_execute(lambda r: 0 if "install" in r.run_id else None)
-    result = proj.run("--root", str(demo_pkg_setuptools), "--develop")
+    result = proj.run("--root", str(demo_pkg_setuptools), "--develop", "--workdir", str(proj.path / ".tox"))
     result.assert_success()
     calls = [(i[0][0].conf.name, i[0][3].run_id) for i in execute_calls.call_args_list]
     expected = [
@@ -430,7 +442,7 @@ def test_sequential_inserted_env_vars(tox_project: ToxProjectCreator, demo_pkg_i
                         k.startswith("TOX_") or k == "VIRTUAL_ENV"]'
     """
     project = tox_project({"tox.ini": ini})
-    result = project.run("r", "--root", str(demo_pkg_inline))
+    result = project.run("r", "--root", str(demo_pkg_inline), "--workdir", str(project.path / ".tox"))
     result.assert_success()
 
     assert re.search(f"TOX_PACKAGE={re.escape(str(project.path))}.*.tar.gz{os.linesep}", result.out)
