@@ -1,11 +1,14 @@
 from __future__ import annotations
 
-from pathlib import Path
 from textwrap import dedent
+from typing import TYPE_CHECKING
 
 import pytest
 
-from tox.pytest import ToxProjectCreator
+if TYPE_CHECKING:
+    from pathlib import Path
+
+    from tox.pytest import ToxProjectCreator
 
 
 @pytest.mark.parametrize(
@@ -167,7 +170,7 @@ def test_pyproject_deps_from_static(
     assert found_calls == expected_calls
 
     if deps:
-        expected_args = ["python", "-I", "-m", "pip", "install"] + deps
+        expected_args = ["python", "-I", "-m", "pip", "install", *deps]
         args = execute_calls.call_args_list[-3][0][3].cmd
         assert expected_args == args
 
@@ -190,7 +193,7 @@ def test_pyproject_deps_from_static(
         ),
     ],
 )
-def test_pyproject_deps_static_with_dynamic(
+def test_pyproject_deps_static_with_dynamic(  # noqa: PLR0913
     tox_project: ToxProjectCreator,
     demo_pkg_inline: Path,
     monkeypatch: pytest.MonkeyPatch,
@@ -210,6 +213,7 @@ def test_pyproject_deps_static_with_dynamic(
     expected_calls = [
         (".pkg", "_optional_hooks"),
         (".pkg", "get_requires_for_build_sdist"),
+        (".pkg", "get_requires_for_build_wheel"),
         (".pkg", "build_wheel"),
         (".pkg", "build_sdist"),
         ("py", "install_package_deps"),
@@ -236,10 +240,10 @@ def test_pyproject_no_build_editable_fallback(tox_project: ToxProjectCreator, de
 
     expected_calls = [
         (".pkg", "_optional_hooks"),
+        (".pkg", "get_requires_for_build_wheel"),
         (".pkg", "build_wheel"),
         (".pkg", "get_requires_for_build_sdist"),
         ("a", "install_package"),
-        (".pkg", "get_requires_for_build_sdist"),
         ("b", "install_package"),
         (".pkg", "_exit"),
     ]
@@ -260,3 +264,34 @@ def test_project_package_with_deps(tox_project: ToxProjectCreator, demo_pkg_setu
     else:
         assert found_calls[0] == (".pkg", "install_requires")
         assert found_calls[1] == (".pkg", "install_deps")
+
+
+def test_pyproject_build_editable_and_wheel(tox_project: ToxProjectCreator, demo_pkg_inline: Path) -> None:
+    # test that build wheel and build editable are cached separately
+
+    ini = """
+    [testenv:.pkg]
+    set_env= BACKEND_HAS_EDITABLE=1
+    [testenv:a,b]
+    package = editable
+    [testenv:c,d]
+    package = wheel
+    """
+    proj = tox_project({"tox.ini": ini}, base=demo_pkg_inline)
+    execute_calls = proj.patch_execute(lambda r: 0 if "install" in r.run_id else None)
+
+    result = proj.run("r", "-e", "a,b,c,d", "--notest", "--workdir", str(proj.path / ".tox"))
+
+    result.assert_success()
+    found_calls = [(i[0][0].conf.name, i[0][3].run_id) for i in execute_calls.call_args_list]
+    assert found_calls == [
+        (".pkg", "_optional_hooks"),
+        (".pkg", "get_requires_for_build_wheel"),
+        (".pkg", "build_editable"),
+        ("a", "install_package"),
+        ("b", "install_package"),
+        (".pkg", "build_wheel"),
+        ("c", "install_package"),
+        ("d", "install_package"),
+        (".pkg", "_exit"),
+    ]
